/*
 *  Vine.cpp - 	 Vine effect
 *               Written by Tinic Urou
 *               Public domain
 */

#include <AppKit.h>
#include <InterfaceKit.h>
#include <KernelKit.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


// Types
typedef unsigned char 	UBYTE;
typedef unsigned short 	UWORD;
typedef unsigned long 	ULONG;
typedef char 			BYTE;
typedef short 			WORD;
typedef long 			LONG;

// Constants

const 	ULONG 			APP_SIGNATURE = '????';
const 	LONG 			DISPLAY_X = 260;
const 	LONG 			DISPLAY_Y = 349;

extern 	UBYTE			pic[];
		LONG			FastCos[260*2];
		LONG			FastSin[349*2];

// Window/Application messages

const 	LONG			MSG_REDRAW = 1;

class 					VineWindow;

// Application object
class Vine : public BApplication 
{
	public:
						Vine();
			void 		ReadyToRun(void);
			bool 		QuitRequested(void);
			void 		AboutRequested(void);

	private:
	
			VineWindow 	*the_window;
			BBitmap 	*the_bitmap;
};

// Window object
class VineWindow : public BWindow 
{
	public:
	
						VineWindow(int width, int height, BBitmap *bitmap);
			bool 		QuitRequested(void);
			void 		MessageReceived(BMessage *msg);

	private:
	
			void		recalc(void);

			BBitmap 	*the_bitmap;
			BView		*main_view;	
			ULONG		xmod;
};

// Create application object and start it
int main(int argc, char **argv)
{	
	Vine *the_app;

	srand(system_time());

	the_app = new Vine();
	the_app->Run();
	delete the_app;

	return 0;
}


/*
 *  Application constructor
 */

Vine::Vine() : BApplication(APP_SIGNATURE)
{
	the_window = NULL;
	the_bitmap = NULL;
}

/*
 *	Thats from me!
 */

void Vine::AboutRequested(void) 
{
	char str[256];
	sprintf(str, "Vine by Tinic Urou\n<5uro@informatik.uni-hamburg.de>\nFreely distributable.");
	BAlert *the_alert = new BAlert("", str, "OK");
	the_alert->Go();
}


/*
 *  Open window and start rendering
 */

void Vine::ReadyToRun(void)
{
	// Allocate the bitmap		
	the_bitmap = new BBitmap(BRect(0, 0, DISPLAY_X-1, DISPLAY_Y-1), B_COLOR_8_BIT);

	// Open window
	the_window = new VineWindow(DISPLAY_X, DISPLAY_Y, the_bitmap);

	// Start action
	the_window->PostMessage(MSG_REDRAW);

	// Let it flow from the orginal to the distorted look

	float x1=0;
	float y1=0;

	for(float g=30.0;g>2.0;g-=(g/30))
	{
		x1=0;
		y1=0;
		
		for (ULONG x=0; x<(DISPLAY_X*2); x++)
		{
			x1+=(((cos(((x*M_PI*8)/DISPLAY_X)))/g)+1);
			FastCos[x]=x1;
		}
		for (ULONG y=0; y<(DISPLAY_Y*2); y++)
		{
			y1+=( ((sin(((y*M_PI*8)/DISPLAY_Y)))/g)+1);
			FastSin[y]=y1;
		}
		snooze(200000);
	}
}


/*
 *  Quit requested (either by menu or by closing the window)
 */

bool Vine::QuitRequested(void)
{
	// Make sure that the window closes first
	if (BApplication::QuitRequested()) 
	{
		if (the_bitmap)	delete the_bitmap;
		return TRUE;
	}
	return FALSE;
}


/*
 *  Window constructor
 */

VineWindow::VineWindow(int width, int height, BBitmap *bitmap) : BWindow(BRect(0, 0, width-1, height-1), "Vine by Tinic", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	the_bitmap = bitmap;
	
	xmod = bitmap->BytesPerRow();
	
	// Move window to right position
	MoveTo(80, 60);

	// Create bitmap view
	Lock();
	main_view = new BView(BRect(0, 0, DISPLAY_X-1, DISPLAY_Y-1),"", B_FOLLOW_NONE,NULL);
	AddChild(main_view);
	Unlock();

	// Show the window
	Show();
}


/*
 *  Closing the window quits the program
 */

bool VineWindow::QuitRequested(void)
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return TRUE;
}


/*
 *  Handles redraw messages
 */

void VineWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case MSG_REDRAW:  // Redraw bitmap
			Lock();
			main_view->DrawBitmapAsync(the_bitmap);
			recalc();
			main_view->Sync();
			PostMessage(MSG_REDRAW);
			Unlock();
			break;
		default:
			BWindow::MessageReceived(msg);
	}
}


/*
 *  Main calculation
 */

void VineWindow::recalc(void)
{
	static 	LONG xo=0,yo=0;

	UBYTE 	*p=pic;
	UBYTE 	*q=(UBYTE *)the_bitmap->Bits();
	LONG	x1,y1;
	
	xo+=2;if(xo>=DISPLAY_X)xo=0;
	yo+=2;if(yo>=DISPLAY_Y)yo=0;
	
	for (ULONG y=0;y<DISPLAY_Y;y++)
	{	
		for (ULONG x=0;x<DISPLAY_X;x++)
		{	
			x1=FastCos[x+xo]-xo;
			y1=FastSin[y+yo]-yo;
	
			if((x1>0)&&(x1<(DISPLAY_X-1))&&(y1>0)&&(y1<DISPLAY_Y))
			{		
				q[x]=p[(y1*DISPLAY_X)+x1];	
			}
			else
			{
				q[x]=0;
			}
		}
		q+=xmod;
	}
}
